
/******************************************************************************************/
// OBJECTS

atom

	var

		// The trigger_req var determines how many triggers targetting this object must be set
		// before the object will activate. The triggers_set var will keep track of how many
		// triggers have been set so far. When a trigger targetting this object is set, the
		// triggers_set var will increase by the trigger_weight of the trigger, and when it is
		// set back the var will decrease by the same amount..
		triggers_req = 1
		triggers_set = 0

		// If trigger_lock is true, then it will lock the object's triggering mechanism as soon
		// as it is either activated or deactivated. Only a manual unlock will allow it to
		// trigger again.
		trigger_lock = 0
		trigger_locked = 0 // true if locked



/******************************************************************************************/
// EXTERNAL USE

// This is called whenever an object is successfully activate by a trigger. It will not
// function if the object is trigger locked.
	proc/Activate(atom/movable/M)
		src.Activated(M)
		return 1


// Called when the object is successfully activated.
	proc/Activated(atom/movable/M)
		return


// This is called whenever an object is dreactived by a trigger. It will not function if
// the object is trigger locked.
	proc/Deactivate(atom/movable/M)
		src.Deactivated(M)
		return 1


// Called when the object is successfully deactivated.
	proc/Deactivated(atom/movable/M)
		return



/******************************************************************************************/
// INTERNAL USE

// Whenever a trigger which is targetting this object is activated, this function will be
// called to add the trigger's trigger weight to the object's triggers_set value. If the
// object's triggers_set equals its triggers_req, then it will activate.
	proc/AddTrigger(atom/movable/M, weight=1)

		// Just in case for some reason someone decided to targe it with a weightless trigger..
		if(!weight)
			return 0

		// If the object's triggers_set equalled its triggers_req before this function
		// was called, then the object was considered active.
		var/was_active = src.triggers_set == src.triggers_req ? 1 : 0
		src.triggers_set += weight
		src.TriggerToggle(M, was_active)
		return 1


// When a trigger which is targetting this object is deactivated, this function is called
// to remove the trigger's trigger weight from the object's triggers_set value. If the
// object's triggers_set value equals its required value, then it will activate
	proc/RemTrigger(atom/movable/M, weight=1)

		// Just in case for some reason someone decided to targe it with a weightless trigger..
		if(!weight)
			return 0

		// If the object's triggers_set equalled its triggers_req before this function
		// was called, then the object was considered active.
		var/was_active = src.triggers_set == src.triggers_req ? 1 : 0
		src.triggers_set -= weight
		src.TriggerToggle(M, was_active)
		return 1


// This function determines whether to activate or deactivate the object based on whether
// triggers_set and triggers_req match now, and whether or not they matched before the
// object's triggers were changed. If the object is activated, it may also lock it.
	proc/TriggerToggle(atom/movable/M, was_active)

		// If this object is locked, nothing will happen past this point.
		if(src.trigger_locked)
			return 0

		// If the object's triggers_set now equals its triggers_req, then it will activate.
		if(src.triggers_set == src.triggers_req)
			src.Activate(M)

			// If it has successfully activated, check to see if it locks when activated, and
			// if so then lock it.
			if(src.trigger_lock)
				src.trigger_locked = 1

		// If the triggers_set doesn't match its triggers_req, check to see if it did before
		// the change. If it didn't match before, nothing will happen, but if it did match
		// before, then the object has been deactivated.
		else
			if(was_active)
				src.Deactivate(M)

		return 1





