
/******************************************************************************************/
// TRIGGERS


trigger
	parent_type = /obj

	var

		// This is a set of one or more params, each param being the tag of an object that this
		// trigger will have an affect on when activated. Example: "gate1;gate2"
		trigger_target = ""

		// If this is true, then the trigger has been activated.
		trigger_active = 0

		// Each object in the game keeps track of how many triggers have been set for it, and
		// this value determines what to adjust the object's number of set triggers by. You can
		// use this in a similar way as bit checks in regular programming.
		trigger_weight = 1

		// The trigger can only be triggered after ever delay time. So if you throw a switch,
		// and the switch has a delay of 10, you'll have to wait 10 ticks before you can throw
		// the switch again.
		trigger_delay = 1

		// A trigger can be equipped with a turn-based timer that will count down each turn. A
		// trigger cannot be deactivated while this counter is set, but once the counter
		// reaches zero the trigger will deactivate itself.
		trigger_timer = 0

		// If this is true, then once the trigger is activated, it cannot be deactivated.
		trigger_permanent = 0

		// These variables work in harmoney with other variables.
		tmp
			last_trigger_time = 0 // keeps track of when the trigger was last used.
			trigger_countdown = 0 // works with the trigger timer.
			permanent_set = 0 // if it has been used once and is not permanently locked



/******************************************************************************************/
// EXTERNAL USE

// Activating a trigger will set its active value to 1 and cause its trigger weight to be
// added to any objects found in its target list.
	Activate(atom/movable/M)

		// If the trigger is locked, it cannot be activated.
		if(src.Locked(M))
			return 0

		src.trigger_active = 1
		src.last_trigger_time = world.time

		// If this is a permanent trigger, then inform the trigger that it has been set.
		if(src.trigger_permanent)
			src.permanent_set = 1

		// Add this trigger's trigger_weight to each object in the trigger's targets list.
		for(var/atom/A in src.AcquireTargets())
			spawn()
				A.AddTrigger(M, src.trigger_weight)

		// Start the countdown timer if applicable.
		src.StartTimer(M)

		// Call the Activated proc for any post-activation effects.
		src.Activated(M)
		return 1


// Called when the trigger is successfully activated.
	Activated(atom/movable/M)
		return


// Deactivating a trigger will set its active value to 0 and cause its trigger weight to
// be removed from any objects found in its target list.
	Deactivate(atom/movable/M)

		// If the trigger is locked, it cannot be activated.
		if(src.Locked(M))
			return 0

		src.trigger_active = 0
		src.last_trigger_time = world.time

		// If this is a permanent trigger, then inform the trigger that it has been set.
		if(src.trigger_permanent)
			src.permanent_set = 1

		// Add this trigger's trigger_weight to each object in the trigger's targets list.
		for(var/atom/A in src.AcquireTargets())
			spawn()
				A.RemTrigger(M, src.trigger_weight)

		// Call the Activated proc for any post-activation effects.
		src.Deactivated(M)
		return 1


// Called when the trigger is successfully deactivated.
	Deactivated(atom/movable/M)
		return


// Toggling a trigger will cause an active trigger to be deactivated, or an inactive
// trigger to be activated. This is typically called for triggers which need to be
// bumped into in order to activate.
	proc/Toggle(atom/movable/M)
		if(src.trigger_active)
			return src.Deactivate(M)
		else
			return src.Activate(M)



/******************************************************************************************/
// TIMER STUFF

// This starts the trigger's count down timer. The countdown can only be started when the
// trigger is activated, as it will deactivate itself when the timer runs out. The
// count down will continue until the counter reaches zero, then it will deactivate.
	proc/StartTimer(atom/movable/M)

		// If this trigger has no countdown timer, or once has already begun, don't do anything.
		if(!src.trigger_timer || src.trigger_countdown)
			return 0

		// Start the timer.
		src.trigger_countdown = src.trigger_timer
		return 1


// Every time this countdown function is called, it will reduce the trigger's counter by
// one until the counter reaches zero, then the trigger will deactivate itself. Countdown
// will only work if the StartTimer function has been called in order to initialize the
// counter. NOTE: You will need to call this function yourself as it is not automatic.
	proc/Countdown(atom/movable/M)

		// If the counter is not counting down, this function doesn't need to do anything.
		if(!src.trigger_countdown)
			return 0

		// If the trigger is not active when this is called, then the counter will be reset
		// to zero since the trigger was somehow deactivated.
		if(!src.trigger_active)
			src.trigger_active = 0
			src.trigger_countdown = 0
			return 0

		src.trigger_countdown--

		// If the counter has been reduced to zero, then we can deactivate the trigger now.
		if(!src.trigger_countdown)
			src.Deactivate(M)
		return 1



/******************************************************************************************/
// INTERNAL USE

// If the trigger is not available to be used at the moment, such as it was activated too
// recently and the trigger_delay variable is dictating that it cannot be interacted with
// again until a set amount of time has passed, then this test will return true, meaning
// that the trigger is locked and you can't do anything with it.
// NOTE that you can override this function if you want to add additional checks to see
// if the trigger is locked or not.
	proc/Locked(atom/movable/A)

		// See if the trigger_delay is saying that this trigger was interacted with too
		// recently, and you'll need to wait to use it again.
		if(src.trigger_delay)
			if(world.time < src.last_trigger_time + src.trigger_delay)
				return 1

		// If the trigger has the trigger_parmanent var set so that it can only be used once,
		// and it has already been used, then the trigger will be considered locked.
		if(src.trigger_permanent && src.permanent_set)
			return 1

		// Triggers can not be used while they are counting down.
		if(src.trigger_countdown)
			return 1

		// Its not locked.
		return 0


// This function retreives all of the objects which own tags specified in the trigger's
// target params, then returns a list containing all of those objects.
	proc/AcquireTargets()
		var/list/target_list = list()

		if(src.trigger_target)
			var/list/targets = params2list(src.trigger_target)
			for(var/ref in targets)

				// Locate the object by matching the tag.
				var/atom/O = locate(ref)
				if(O)
					target_list += O

		return target_list









